@@ -90,6 +90,10 @@ type LStreamClient struct {
 
 	conn *connCtx
 
+	// connDebugMessages contains human-readable messages received from the shell
+	// transport regarding the connection.
+	connDebugMessages []string
+
 	cmdQueue   []lstreamCmd
 	curCmdCtx  *lstreamCmdCtx
 	nextCmdIdx int
@@ -150,8 +154,16 @@ type BusyStage struct {
 }
 
 type ConnDetails struct {
+	// Messages contains human-readable messages received from shell transport
+	// regarding the connection.
+	Messages []string
+
 	// Err is an error message from the last connection attempt.
 	Err string
+
+	// Connected shows whether the connection has already succeeded. Unlike other
+	// fields in this struct, it's set by the LStreamsManager manually.
+	Connected bool
 }
 
 type BootstrapDetails struct {
@@ -233,14 +245,28 @@ func createTransport(
 ) ShellTransport {
 	var transport ShellTransport
 
-	if config.SSH != nil {
+	if config.SSHLib != nil {
 		if transport != nil {
 			panic("transport config is ambiguous")
 		}
 
-		transport = NewShellTransportSSH(ShellTransportSSHParams{
+		transport = NewShellTransportSSHLib(ShellTransportSSHLibParams{
 			SSHKeys:     sshKeys,
-			ConnDetails: *config.SSH,
+			ConnDetails: *config.SSHLib,
+
+			Logger: logger,
+		})
+	}
+
+	if config.SSHBin != nil {
+		if transport != nil {
+			panic("transport config is ambiguous")
+		}
+
+		transport = NewShellTransportSSHBin(ShellTransportSSHBinParams{
+			Host: config.SSHBin.Host,
+			User: config.SSHBin.User,
+			Port: config.SSHBin.Port,
 
 			Logger: logger,
 		})
@@ -350,6 +376,8 @@ func (lsc *LStreamClient) changeState(newState LStreamClientState) {
 	case LStreamClientStateConnecting:
 		// Forget whatever queued command we might have.
 		lsc.cmdQueue = nil
+		// Forget whatever conn debug messages we've accumulated.
+		lsc.connDebugMessages = nil
 
 		// Initiate new connection
 		lsc.numConnAttempts++
@@ -369,6 +397,13 @@ func (lsc *LStreamClient) changeState(newState LStreamClientState) {
 	}
 }
 
+func (lsc *LStreamClient) makeConnDetailsMsg(err string) *ConnDetails {
+	return &ConnDetails{
+		Messages: lsc.connDebugMessages,
+		Err:      err,
+	}
+}
+
 func (lsc *LStreamClient) sendBusyStageUpdate() {
 	upd := lsc.busyStage
 	lsc.sendUpdate(&LStreamClientUpdate{
@@ -400,7 +435,14 @@ func (lsc *LStreamClient) run() {
 	for {
 		select {
 		case upd := <-lsc.connectUpdCh:
-			if dataReq := upd.DataRequest; dataReq != nil {
+			if dbg := upd.DebugInfo; dbg != nil {
+				// Got some debug info about the connection.
+				lsc.connDebugMessages = append(lsc.connDebugMessages, dbg.Message)
+
+				lsc.sendUpdate(&LStreamClientUpdate{
+					ConnDetails: lsc.makeConnDetailsMsg(""),
+				})
+			} else if dataReq := upd.DataRequest; dataReq != nil {
 				// We need some data from the user.
 
 				lsc.sendUpdate(&LStreamClientUpdate{
@@ -412,9 +454,7 @@ func (lsc *LStreamClient) run() {
 				if res.Err != nil {
 					lsc.params.Logger.Errorf("Shell connection failed: %s", res.Err.Error())
 					lsc.sendUpdate(&LStreamClientUpdate{
-						ConnDetails: &ConnDetails{
-							Err: fmt.Sprintf("attempt %d: %s", lsc.numConnAttempts, res.Err.Error()),
-						},
+						ConnDetails: lsc.makeConnDetailsMsg(fmt.Sprintf("attempt %d: %s", lsc.numConnAttempts, res.Err.Error())),
 					})
 
 					lsc.changeState(LStreamClientStateDisconnected)
@@ -1648,7 +1688,31 @@ func parseCommandDoneLine(line string, expectedIdx int) (*commandDoneDetails, er
 	}, nil
 }
 
+// requiresShellQuoting returns whether the given string requires to be quoted
+// in a shell.
+func requiresShellQuoting(s string) bool {
+	// Empty string requires quoting
+	if len(s) == 0 {
+		return true
+	}
+
+	for _, r := range s {
+		if !((r >= 'a' && r <= 'z') ||
+			(r >= 'A' && r <= 'Z') ||
+			(r >= '0' && r <= '9') ||
+			r == '_' || r == '.' || r == '/' || r == '-') {
+			return true
+		}
+	}
+
+	return false
+}
+
 func shellQuote(s string) string {
+	if !requiresShellQuoting(s) {
+		return s
+	}
+
 	return fmt.Sprintf("'%s'", strings.Replace(s, "'", "'\"'\"'", -1))
 }
 
