@@ -26,8 +26,9 @@ type LStreamsManager struct {
 
 	lscs      map[string]*LStreamClient
 	lscStates map[string]LStreamClientState
-	// lscConnDetails only contains items for lstreams which are in the
-	// LStreamClientStateConnectinConnecting state.
+	// lscConnDetails contains items for all selected lstreams, even after the
+	// connection is done (which is indicated by ConnDetails.Connected being
+	// true).
 	lscConnDetails map[string]ConnDetails
 	// lscBusyStages only contains items for lstreams which are in the
 	// LStreamClientStateConnectedBusy state.
@@ -57,6 +58,8 @@ type LStreamsManager struct {
 	curQueryLogsCtx *manQueryLogsCtx
 
 	curLogs manLogsCtx
+
+	useExternalSSH bool
 }
 
 type LStreamsManagerParams struct {
@@ -76,6 +79,8 @@ type LStreamsManagerParams struct {
 
 	InitialLStreams string
 
+	InitialUseExternalSSH bool
+
 	// ClientID is just an arbitrary string (should be filename-friendly though)
 	// which will be appended to the nerdlog_agent.sh and its index filenames.
 	//
@@ -112,6 +117,8 @@ func NewLStreamsManager(params LStreamsManagerParams) *LStreamsManager {
 
 		teardownReqCh: make(chan struct{}, 1),
 		torndownCh:    make(chan struct{}, 1),
+
+		useExternalSSH: params.InitialUseExternalSSH,
 	}
 
 	if err := lsman.setLStreams(params.InitialLStreams); err != nil {
@@ -127,6 +134,41 @@ func NewLStreamsManager(params LStreamsManagerParams) *LStreamsManager {
 	return lsman
 }
 
+func (lsman *LStreamsManager) SetUseExternalSSH(useExternalSSH bool) {
+	resCh := make(chan struct{}, 1)
+
+	lsman.reqCh <- lstreamsManagerReq{
+		setUseExternalSSH: &lstreamsManagerReqSetUseExternalSSH{
+			useExternalSSH: useExternalSSH,
+			resCh:          resCh,
+		},
+	}
+
+	<-resCh
+}
+
+func (lsman *LStreamsManager) setUseExternalSSH(useExternalSSH bool) {
+	// If unchanged, then do nothing.
+	if lsman.useExternalSSH == useExternalSSH {
+		return
+	}
+
+	// Transport mode has changed: remember it, and reconnect using it.
+
+	lsman.useExternalSSH = useExternalSSH
+
+	lstreamsStr := lsman.lstreamsStr
+	lsman.setLStreams("")
+	lsman.updateHAs()
+	lsman.updateLStreamsByState()
+
+	lsman.setLStreams(lstreamsStr)
+	lsman.updateHAs()
+	lsman.updateLStreamsByState()
+
+	lsman.sendStateUpdate()
+}
+
 func (lsman *LStreamsManager) setLStreams(lstreamsStr string) error {
 	u, err := user.Current()
 	if err != nil {
@@ -136,6 +178,8 @@ func (lsman *LStreamsManager) setLStreams(lstreamsStr string) error {
 	resolver := NewLStreamsResolver(LStreamsResolverParams{
 		CurOSUser: u.Username,
 
+		UseExternalSSH: lsman.useExternalSSH,
+
 		ConfigLogStreams: lsman.params.ConfigLogStreams,
 		SSHConfig:        lsman.params.SSHConfig,
 	})
@@ -216,7 +260,9 @@ func (lsman *LStreamsManager) run() {
 					// Maintain lsman.lscConnDetails
 					if upd.State.NewState == LStreamClientStateConnectedIdle ||
 						upd.State.NewState == LStreamClientStateConnectedBusy {
-						delete(lsman.lscConnDetails, upd.Name)
+						cd := lsman.lscConnDetails[upd.Name]
+						cd.Connected = true
+						lsman.lscConnDetails[upd.Name] = cd
 					}
 
 					// Maintain lsman.lscBusyStages
@@ -286,7 +332,7 @@ func (lsman *LStreamsManager) run() {
 							break
 						}
 
-						if i > 0 {
+						if pendingSB.Len() > 0 {
 							pendingSB.WriteString(", ")
 						}
 
@@ -414,6 +460,14 @@ func (lsman *LStreamsManager) run() {
 
 				r.resCh <- nil
 
+			case req.setUseExternalSSH != nil:
+				r := req.setUseExternalSSH
+				lsman.params.Logger.Infof("LStreams manager: setting useExternalSSH: %v", r.useExternalSSH)
+
+				lsman.setUseExternalSSH(r.useExternalSSH)
+
+				r.resCh <- struct{}{}
+
 			case req.ping:
 				for _, lsc := range lsman.lscs {
 					lsc.EnqueueCmd(lstreamCmd{
@@ -570,11 +624,12 @@ func (lsman *LStreamsManager) Wait() {
 type lstreamsManagerReq struct {
 	// Exactly one field must be non-nil
 
-	queryLogs   *QueryLogsParams
-	updLStreams *lstreamsManagerReqUpdLStreams
-	ping        bool
-	reconnect   bool
-	disconnect  bool
+	queryLogs         *QueryLogsParams
+	updLStreams       *lstreamsManagerReqUpdLStreams
+	setUseExternalSSH *lstreamsManagerReqSetUseExternalSSH
+	ping              bool
+	reconnect         bool
+	disconnect        bool
 }
 
 type lstreamsManagerReqUpdLStreams struct {
@@ -582,6 +637,11 @@ type lstreamsManagerReqUpdLStreams struct {
 	resCh          chan<- error
 }
 
+type lstreamsManagerReqSetUseExternalSSH struct {
+	useExternalSSH bool
+	resCh          chan<- struct{}
+}
+
 func (lsman *LStreamsManager) QueryLogs(params QueryLogsParams) {
 	lsman.params.Logger.Verbose1f("QueryLogs: %+v", params)
 	lsman.reqCh <- lstreamsManagerReq{
