@@ -547,19 +547,34 @@ function run_awk_script_logfiles {
   }
   '$awk_pattern'
   {
-    # Account for decreased timestamps.
-    #
-    # NOTE: to make it produce the correct result in all cases, this check
-    # needs to be before the pattern check, but we intentionally avoid doing
-    # that because it slows things down by 5-10% when the pattern filters out
-    # most of the lines, which I think is not worth it to account for this
-    # corner case.
     curMinKey = '"$awktime_minute_key"';
-    if (curMinKey < prevMinKey) {
-      curMinKey = prevMinKey;
-    } else {
-      prevMinKey = curMinKey;
-    }
+
+    # NOTE: this was a naive attempt to better handle the case when timestamps
+    # have decreased: instead of incrementing the bucket of the decreased
+    # timestamp, we ideally want to increment the bucket of the last
+    # non-decreased timestamp.
+    #
+    # However, to make it work properly, the minute key needs to be formatted
+    # so that a later timestamp is always lexocographically larger than an
+    # earlier timestamp, and while it is possible to implement it this way,
+    # it slows things down significantly, which seems unjustified just to
+    # handle this corner case more gracefully.
+    #
+    # We might still implement it at some point and make it optional, but for
+    # now, keeping things simple and just not caring about this corner case.
+    #
+    ## Account for decreased timestamps.
+    ##
+    ## NOTE: to make it produce the correct result in all cases, this check
+    ## needs to be before the pattern check, but we intentionally avoid doing
+    ## that because it slows things down by 5-10% when the pattern filters out
+    ## most of the lines, which I think is not worth it to account for this
+    ## corner case.
+    #if (curMinKey < prevMinKey) {
+      #curMinKey = prevMinKey;
+    #} else {
+      #prevMinKey = curMinKey;
+    #}
 
     stats[curMinKey]++;
 
